import { Server, Socket } from "net";
import { PushReadableStream, UnwrapConsumableStream, WrapWritableStream, WritableStream, } from "@yume-chan/stream-extra";
function nodeSocketToStreamPair(socket) {
    socket.setNoDelay(true);
    return {
        readable: new PushReadableStream((controller) => {
            // eslint-disable-next-line @typescript-eslint/no-misused-promises
            socket.on("data", async (data) => {
                socket.pause();
                await controller.enqueue(data);
                socket.resume();
            });
            socket.on("end", () => {
                try {
                    controller.close();
                }
                catch (e) {
                    // controller already closed
                }
            });
            controller.abortSignal.addEventListener("abort", () => {
                socket.end();
            });
        }),
        writable: new WritableStream({
            write: async (chunk) => {
                await new Promise((resolve, reject) => {
                    socket.write(chunk, (err) => {
                        if (err) {
                            reject(err);
                        }
                        else {
                            resolve();
                        }
                    });
                });
            },
            close() {
                return new Promise((resolve) => {
                    socket.end(resolve);
                });
            },
        }),
    };
}
export class AdbServerNodeTcpConnection {
    spec;
    #listeners = new Map();
    constructor(spec) {
        this.spec = spec;
    }
    async connect({ unref } = { unref: false }) {
        const socket = new Socket();
        if (unref) {
            socket.unref();
        }
        socket.connect(this.spec);
        await new Promise((resolve, reject) => {
            socket.once("connect", resolve);
            socket.once("error", reject);
        });
        return nodeSocketToStreamPair(socket);
    }
    async addReverseTunnel(handler, address) {
        // eslint-disable-next-line @typescript-eslint/no-misused-promises
        const server = new Server(async (socket) => {
            const stream = nodeSocketToStreamPair(socket);
            try {
                await handler({
                    service: address,
                    readable: stream.readable,
                    writable: new WrapWritableStream(stream.writable).bePipedThroughFrom(new UnwrapConsumableStream()),
                    close() {
                        socket.end();
                    },
                });
            }
            catch {
                socket.end();
            }
        });
        if (address) {
            const url = new URL(address);
            if (url.protocol === "tcp:") {
                server.listen(Number.parseInt(url.port, 10), url.hostname);
            }
            else if (url.protocol === "unix:") {
                server.listen(url.pathname);
            }
            else {
                throw new Error(`Unsupported protocol ${url.protocol}`);
            }
        }
        else {
            server.listen();
        }
        await new Promise((resolve, reject) => {
            server.on("listening", () => resolve());
            server.on("error", (e) => reject(e));
        });
        if (!address) {
            const info = server.address();
            address = `tcp:${info.address}:${info.port}`;
        }
        this.#listeners.set(address, server);
        return address;
    }
    removeReverseTunnel(address) {
        const server = this.#listeners.get(address);
        if (!server) {
            return;
        }
        server.close();
        this.#listeners.delete(address);
    }
    clearReverseTunnels() {
        for (const server of this.#listeners.values()) {
            server.close();
        }
        this.#listeners.clear();
    }
}
//# sourceMappingURL=index.js.map